class BaseConfig {
    // Grid
    public int grid_y; // grid columns
    public int grid_x; // grid rows
    public int grid_row_spacing;
    public int grid_col_spacing;
    // AppItem
    public double item_font_size;
    public int item_icon_size;
    public int item_box_width;
    public int item_box_height;
    // SearchBar
    public int sb_width;
    public int sb_height;

    public int screen_w;
    public int screen_h;

    public BaseConfig(int screen_width, int screen_height, bool init_default = true) {
        screen_w = screen_width;
        screen_h = screen_height;
        if (init_default) {
            grid_row_spacing = 24;
            grid_col_spacing = 16;
        
            default_grid_size();
            
            item_icon_size = -1;
            while (item_icon_size < 0) {
                default_icon_size();
            }

            item_font_size = 12;

            item_box_width = item_icon_size * 2;
            item_box_height = item_icon_size * 2;
        }
    }
    
    private void default_icon_size() {
        int[] default_icon_sizes = { 24, 32, 48, 64, 96 };
        
        int best_size = -1;
        int indicator_h = 60;
        int search_h = 96;
        
        foreach (int current_icon_size in default_icon_sizes) {
            double vertical_size = (grid_x * ((current_icon_size * 2) + grid_row_spacing)) + indicator_h + search_h + 20;
            double horizontal_size = grid_y * ((current_icon_size * 2) + grid_col_spacing);
        
            if ((screen_w > horizontal_size) && (screen_h > vertical_size)) {
                best_size = current_icon_size;
            } else {
                break;
            }
        }
        if (best_size < 0) {
            double vertical_size = (grid_x * ((default_icon_sizes[0] * 2) + grid_row_spacing)) + indicator_h + search_h + 20;
            double horizontal_size = grid_y * ((default_icon_sizes[0] * 2) + grid_col_spacing);
            
            while (screen_w < horizontal_size) {
                int new_grid_y = grid_y - 1;
                if (new_grid_y < 1) {
                    best_size = default_icon_sizes[0];
                    break;
                } else {
                    grid_y = new_grid_y;
                    horizontal_size = grid_y * ((default_icon_sizes[0] * 2) + grid_col_spacing);
                }
            }
            while (screen_h < vertical_size) {
                int new_grid_x = grid_x - 1;
                if (new_grid_x < 1) {
                    best_size = default_icon_sizes[0];
                    break;
                } else {
                    grid_x = new_grid_x;
                    vertical_size = (grid_x * ((default_icon_sizes[0] * 2) + grid_row_spacing)) + indicator_h + search_h + 20;
                }
            }
        }
        
        this.item_icon_size = best_size;
    }
    
    private void default_grid_size() {
        int[] default_grid_x_sizes = { 8, 3, 6, 4 };
        int[] default_grid_y_sizes = { 3, 8, 4, 6 };
        int num_default_grid_sizes = 4;
        
        double screen_ratio = (screen_w / (double) screen_h);
        double closest_ratio = double.INFINITY;
        int best_index = -1;
        
        int i;
        for (i = 0; i < num_default_grid_sizes; i++) {
            double grid_ratio = (default_grid_y_sizes[i] / (double) default_grid_x_sizes[i]);
            
            if ((screen_ratio - grid_ratio).abs() < (screen_ratio - closest_ratio).abs()) {
                closest_ratio = grid_ratio;
                best_index = i;
            }
        }
        
        grid_x = default_grid_x_sizes[best_index];
        grid_y = default_grid_y_sizes[best_index];
    }
}

void merge_int(int* ptr, int val) {
    if (val > -1)
        *ptr = val;
}

void merge_double(double* ptr, double val) {
    if (val > -1.0)
        *ptr = val;
}

class FileConfig : BaseConfig {
    private KeyFile config_f;

    public FileConfig(int screen_width, int screen_height, string file) {
        base(screen_width, screen_height);

        config_f = new KeyFile();
        try {
            config_f.load_from_file(file, KeyFileFlags.KEEP_COMMENTS);
        } catch {
            message ("Config file not found. Using default values");
            return;
        }
        
        const string[] group = {"Grid", "AppItem", "SearchBar"};
        try {
            merge_int(&grid_y, config_f.get_integer(group[0], "Y"));
            merge_int(&grid_x, config_f.get_integer(group[0], "X"));
            merge_int(&grid_row_spacing, config_f.get_integer(group[0], "RowSpacing"));
            merge_int(&grid_col_spacing, config_f.get_integer(group[0], "ColumnSpacing"));

            merge_double(&item_font_size, config_f.get_double(group[1], "FontSize"));
            merge_int(&item_icon_size, config_f.get_integer(group[1], "IconSize"));
            merge_int(&item_box_width, config_f.get_integer(group[1], "BoxWidth"));
            merge_int(&item_box_height, config_f.get_integer(group[1], "BoxHeight"));

            merge_int(&sb_width, config_f.get_integer(group[2], "Width"));
            merge_int(&sb_height, config_f.get_integer(group[2], "Height"));
        }
        catch {
            message ("Key config missing");
        }
    }
}

        //  grid_y              = config_f.get_integer(group[0], "Y");
        //  grid_x              = config_f.get_integer(group[0], "X");
        //  grid_row_spacing = config_f.get_integer(group[0], "RowSpacing");
        //  grid_col_spacing = config_f.get_integer(group[0], "ColumnSpacing");
        //  item_font_size      = config_f.get_double(group[1], "FontSize");
        //  item_icon_size      = config_f.get_integer(group[1], "IconSize");
        //  item_box_width      = config_f.get_integer(group[1], "BoxWidth");
        //  item_box_height     = config_f.get_integer(group[1], "BoxHeight");
        //  sb_width            = config_f.get_integer(group[2], "Width");
        //  sb_height           = config_f.get_integer(group[2], "Height");
