/*
* Copyright (c) 2011-2020 LightPad
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/

using GLib;
using Gtk;
using Cairo;

namespace ZorinAppGridLite.Frontend {

    class Utilities : GLib.Object {
   
        public static ZorinAppGridLite.Frontend.Color average_color (Gdk.Pixbuf source) {
            double rTotal = 0;
            double gTotal = 0;
            double bTotal = 0;

            uchar* dataPtr = source.get_pixels ();
            double pixels = source.height * source.rowstride / source.n_channels;

            for (int i = 0; i < pixels; i++) {
                uchar r = dataPtr [0];
                uchar g = dataPtr [1];
                uchar b = dataPtr [2];

                uchar max = (uchar) Math.fmax (r, Math.fmax (g, b));
                uchar min = (uchar) Math.fmin (r, Math.fmin (g, b));
                double delta = max - min;

                double sat = delta == 0 ? 0 : delta / max;
                double score = 0.2 + 0.8 * sat;

                rTotal += r * score;
                gTotal += g * score;
                bTotal += b * score;
                
                dataPtr += source.n_channels;
            }

            return ZorinAppGridLite.Frontend.Color (rTotal / uint8.MAX / pixels,
                        gTotal / uint8.MAX / pixels,
                        bTotal / uint8.MAX / pixels,
                        1).set_val (0.8).multiply_sat (1.15);
        }
    }

}
