/*
* Copyright (c) 2011-2020 LightPad
*               2021 Zorin OS Technologies Ltd.
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* General Public License for more details.
*
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
* Boston, MA 02110-1301 USA
*/

namespace ZorinAppGridLite.Frontend {

    public class Indicators : Gtk.HBox {
    
        // Signals
        public signal void child_activated ();

        // Properties
        public new GLib.List<Gtk.Widget> children;
        public int active = -1;
        private int old_active = -1;
        private int skip_flag = 0;
        
        public Indicators () {
            this.homogeneous = false;
            this.spacing = 0;
        }

        public void append (string thelabel) {
            var indicator = new Gtk.EventBox ();
            indicator.set_visible_window (false);

            var label = new Gtk.Label (thelabel);
            label.get_style_context ().add_class ("indicator_label");

            // make sure the child widget is added with padding
            label.set_halign (Gtk.Align.CENTER);
            label.set_valign (Gtk.Align.CENTER);
            label.set_margin_top (5);
            label.set_margin_bottom (5);
            label.set_margin_start (15);
            label.set_margin_end (15);
            indicator.add (label);
            this.children.append (indicator);
            
            indicator.button_release_event.connect( () => {
                this.set_active (this.children.index (indicator));
                return true;
            });

            this.pack_start (indicator, false, false, 0);
        }
        
        public void set_active_no_signal (int index) {
            int pages_length = (int) this.children.length ();
            // make sure the requested active item is in the children list
            if (index <= (pages_length - 1)) {
                this.old_active = this.active;
                this.active = index;
                this.change_focus ();
            }

        }

        public void set_active (int index) {
            skip_flag++;
            this.set_active_no_signal (index);
            if (skip_flag > 1) { // avoid activating a page "0" that does not exist
                this.child_activated (); // send signal
            }
        }
        
        public void change_focus () {
            Gtk.Widget old_indicator = this.get_children ().nth_data (this.old_active);
            if (old_indicator != null) {
                old_indicator.get_style_context ().remove_class ("active");
            }
            
            Gtk.Widget new_indicator = this.get_children ().nth_data (this.active);
            if (old_indicator != null) {
                new_indicator.get_style_context ().add_class ("active");
            }
        }
    }

}
